/*
 * Copyright (C) 2005 ruby-ecore team (see AUTHORS)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <ruby.h>

#include <Ecore.h>
#include <Ecore_Evas.h>

#include "../ecore/rb_ecore.h"
#include "../ecore_x/rb_window.h"
#include "rb_ecore_evas_main.h"
#include "rb_ecore_evas.h"

typedef struct {
	RbEcoreEvas ee;
	VALUE parent_window;
	VALUE window;
} RbEcoreEvasXRenderX11;

static void c_mark (RbEcoreEvasXRenderX11 *ee)
{
	c_ecore_evas_mark (&ee->ee);

	if (!NIL_P (ee->parent_window))
		rb_gc_mark (ee->parent_window);

	if (!NIL_P (ee->window))
		rb_gc_mark (ee->window);
}

static void c_free (RbEcoreEvasXRenderX11 *ee)
{
	c_ecore_evas_free (&ee->ee, false);

	free (ee);
}

static VALUE c_alloc (VALUE klass)
{
	RbEcoreEvasXRenderX11 *ee = NULL;

	ecore_init ();
	ecore_evas_init ();

	return Data_Make_Struct (klass, RbEcoreEvasXRenderX11,
	                         c_mark, c_free, ee);
}

/*
 * call-seq:
 *  Ecore::Evas::XRenderX11.new([display, parent, x, y, w, h]) => xrx11
 *
 * Creates an Ecore::Evas::XRenderX11 object.
 */
static VALUE c_init (int argc, VALUE *argv, VALUE self)
{
	VALUE disp, parent, geom[4];
	RbWindow *win = NULL;
	char *cdisp = NULL;
	int i, igeom[4] = {0, 0, 0, 0};

	GET_OBJ (self, RbEcoreEvasXRenderX11, ee);

	rb_scan_args (argc, argv, "06", &disp, &parent,
	              &geom[0], &geom[1], &geom[2], &geom[3]);

	if (!NIL_P (disp)) {
		Check_Type (disp, T_STRING);
		cdisp = StringValuePtr (disp);
	}

	if (!NIL_P (parent)) {
		CHECK_CLASS (parent, cWindow);
		Data_Get_Struct (parent, RbWindow, win);
	}

	for (i = 0; i < 4; i++)
		if (!NIL_P (geom[i])) {
			Check_Type (geom[i], T_FIXNUM);
			igeom[i] = FIX2INT (geom[i]);
		}

	ee->ee.real = ecore_evas_xrender_x11_new (cdisp,
	                                          win ? win->real : 0,
	                                          igeom[0], igeom[1],
	                                          igeom[2], igeom[3]);
	ee->parent_window = parent;
	ee->window = Qnil;

	rb_call_super (argc, argv);

	return self;
}

/*
 * call-seq:
 *  xrx11.window => window
 *
 * Returns the <code>Ecore::X::Window</code> object for <i>xrx11</i>.
 */
static VALUE c_window_get (VALUE self)
{
	Ecore_X_Window w;

	GET_OBJ (self, RbEcoreEvasXRenderX11, ee);

	if (NIL_P (ee->window)) {
		w = ecore_evas_xrender_x11_window_get (ee->ee.real);
		ee->window = TO_ECORE_X_WINDOW (self, w);
	}

	return ee->window;
}

void Init_XRenderX11 (void)
{
	VALUE c;

	c = rb_define_class_under (mEvas, "XRenderX11", cEcoreEvas);

	rb_define_alloc_func (c, c_alloc);
	rb_define_method (c, "initialize", c_init, -1);
	rb_define_method (c, "window", c_window_get, 0);
}
